<?php

namespace FiloBlu\CheckoutExtend\Api\Plugin\Sales;

use Magento\Quote\Model\Quote\Address as QuoteAddress;
use Magento\Framework\App\ObjectManager;

class OrderManagementInterface {

    protected $_subscriberFactory;
    protected $_helper;
    protected $_logger;
    protected $_quoteFactory;
    protected $_request;
    protected $_checkoutSession;
    protected $objectCopyService;
    protected $addressFactory;
    protected $regionFactory;
    protected $customerFactory;
    protected $session;
    protected $accountManagement;
    protected $cookieMetadataManager;
    protected $cookieMetadataFactory;
    protected $orderRepository;
    protected $_eventManager;

    public function __construct(
    \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory, \FiloBlu\CheckoutExtend\Helper\Data $helper, \Magento\Quote\Model\QuoteFactory $quoteFactory, \Magento\Framework\App\RequestInterface $request, \Magento\Checkout\Model\Session $checkoutSession, \Magento\Framework\DataObject\Copy $objectCopyService, \Magento\Customer\Api\Data\AddressInterfaceFactory $addressFactory, \Magento\Customer\Api\Data\RegionInterfaceFactory $regionFactory, \Magento\Customer\Api\Data\CustomerInterfaceFactory $customerFactory, \Magento\Customer\Model\Session $customerSession, \Magento\Customer\Api\AccountManagementInterface $accountManagement, \Magento\Sales\Api\OrderRepositoryInterface $orderRepository, \Magento\Framework\Event\ManagerInterface $eventManager
    ) {
        $this->_subscriberFactory = $subscriberFactory;
        $this->_helper = $helper;
        $this->_quoteFactory = $quoteFactory;
        $this->_request = $request;
        $this->_checkoutSession = $checkoutSession;
        $this->objectCopyService = $objectCopyService;
        $this->addressFactory = $addressFactory;
        $this->regionFactory = $regionFactory;
        $this->customerFactory = $customerFactory;
        $this->session = $customerSession;
        $this->accountManagement = $accountManagement;
        $this->orderRepository = $orderRepository;
        $this->_eventManager = $eventManager;
    }

    public function afterPlace($interceptor, $order) {

        $quote = null;

        if ($order->getQuoteId()) {
            $quote = $this->_quoteFactory->create()->load($order->getQuoteId());
        }

        // Create Customer Account
        try {
            if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/registration/enabled") && !$order->getCustomerId() && !$this->session->isLoggedIn() && $quote && $quote->getCheckoutRegistration()) {

//                $this->session->regenerateId();

                $customerData = $this->objectCopyService->copyFieldsetToTarget(
                        'order_address', 'to_customer', $order->getBillingAddress(), []
                );

                $addresses = $order->getAddresses();
                foreach ($addresses as $address) {
                    $addressData = $this->objectCopyService->copyFieldsetToTarget(
                            'order_address', 'to_customer_address', $address, []
                    );
                    /** @var \Magento\Customer\Api\Data\AddressInterface $customerAddress */
                    $customerAddress = $this->addressFactory->create(['data' => $addressData]);
                    switch ($address->getAddressType()) {
                        case QuoteAddress::ADDRESS_TYPE_BILLING:
                            $customerAddress->setIsDefaultBilling(true);
                            break;
                        case QuoteAddress::ADDRESS_TYPE_SHIPPING:
                            $customerAddress->setIsDefaultShipping(true);
                            break;
                    }

                    if (is_string($address->getRegion())) {
                        /** @var \Magento\Customer\Api\Data\RegionInterface $region */
                        $region = $this->regionFactory->create();
                        $region->setRegion($address->getRegion());
                        $region->setRegionCode($address->getRegionCode());
                        $region->setRegionId($address->getRegionId());
                        $customerAddress->setRegion($region);
                    }
                    $customerData['addresses'][] = $customerAddress;
                }

                $customerPwd = $this->_checkoutSession->getRegistrationPwd();

                $customer = $this->customerFactory->create(['data' => $customerData]);
                $account = $this->accountManagement->createAccount($customer, $customerPwd);

                $this->_eventManager->dispatch(
                        'customer_register_success', ['account_controller' => null, 'customer' => $account]
                );

                if ($account->getId()) {
                    $order->setCustomerId((int) $account->getId());
                    $this->orderRepository->save($order);
                    $this->session->setCustomerMustLogin((int) $account->getId());
                }

                $this->_checkoutSession->setRegistrationPwd(null);
            }
        } catch (\Exception $ex) {
            
        }

        // Subscribe to Newsletter
        try {


            if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/newsletter/enabled") && $quote && $quote->getIsSubscribeForNewsletter()) {

                $shippingAddress = $quote->getShippingAddress();

                if ($quote->getCustomerIsGuest()) {

                    // Set newsletter subscriber data in request
                    $this->_request->setPostValue('firstname', $shippingAddress->getFirstname());
                    $this->_request->setPostValue('lastname', $shippingAddress->getLastname());
                    $this->_request->setPostValue('country_id', $shippingAddress->getCountryId());
                    $this->_request->setPostValue('phone', $shippingAddress->getTelephone());
                    $this->_request->setPostValue('source', "CHECKOUT");
                    $this->_request->setPostValue('language', $this->_helper->getCurrentLocale());
                    $this->_request->setPostValue('website_name', $this->_helper->getCurrentWebsiteName());

                    // Fields not set: dob - gender - collections - birthday

                    $this->_subscriberFactory->create()->subscribe($quote->getCustomerEmail());
                } else {

                    // Set newsletter subscriber data in request
                    $this->_request->setPostValue('firstname', ($quote->getCustomerFirstname() ?: $shippingAddress->getFirstname()));
                    $this->_request->setPostValue('lastname', ($quote->getCustomerLastname() ?: $shippingAddress->getLastname()));
                    $this->_request->setPostValue('country_id', $shippingAddress->getCountryId());
                    $this->_request->setPostValue('phone', $shippingAddress->getTelephone());
                    $this->_request->setPostValue('source', "CHECKOUT");
                    $this->_request->setPostValue('language', $this->_helper->getCurrentLocale());
                    $this->_request->setPostValue('website_name', $this->_helper->getCurrentWebsiteName());

                    // Fields not set: dob - gender - collections - birthday

                    $this->_subscriberFactory->create()->subscribeCustomerById((int) $order->getCustomerId());
                }
            }
        } catch (\Exception $ex) {
            
        }


        return $order;
    }

    private function getCookieManager() {
        if (!$this->cookieMetadataManager) {
            $this->cookieMetadataManager = ObjectManager::getInstance()->get(
                    \Magento\Framework\Stdlib\Cookie\PhpCookieManager::class
            );
        }
        return $this->cookieMetadataManager;
    }

    private function getCookieMetadataFactory() {
        if (!$this->cookieMetadataFactory) {
            $this->cookieMetadataFactory = ObjectManager::getInstance()->get(
                    \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory::class
            );
        }
        return $this->cookieMetadataFactory;
    }

}
